#ifndef _2D_PHYSICS_OBJECT_H_
#define _2D_PHYSICS_OBJECT_H_

#include <string>
#include <list>

#include "colliders.h"
#include "rigidbody2D.h"
#include "vector2.h"

using std::string;

enum class collision_position{ TOP, BOTTOM, LEFT, RIGHT };

class PhysicsObject2D
{
private:
	struct CollisionData
	{
		string collisionTag;
		collision_position collisionPosition;
	};

public:
	PhysicsObject2D();
	PhysicsObject2D(const PhysicsObject2D&);
	~PhysicsObject2D();

	bool Initialize(Vector2, float mass, string tag, int radius);
	bool Initialize(Vector2, float mass, string tag, int width, int height);

	void Shutdown();

	inline Vector2 GetPosition()
	{
		return m_position;
	}

	inline void SetPosition(Vector2 position)
	{
		m_position = position;
	}

	inline Collider2D* GetCollider()
	{
		return m_collider;
	}

	inline Rigidbody2D* GetRigidbody()
	{
		return m_rigidbody;
	}

	inline Vector2 GetPotentialPosition()
	{
		return _potentialPosition;
	}

	inline void SetPotentialPosition(Vector2 position)
	{
		_potentialPosition = position;
	}

	inline void UpdateRotation(float rotation)
	{
		_rotation += rotation;
		if (_rotation >= 360)
		{
			_rotation -= 360;
		}
		else if (_rotation < 0)
		{
			_rotation += 360;
		}
	}

	inline float GetRotation()
	{
		return _rotation;
	}

	inline void SetObjectTag(string tag)
	{
		_objectTag = tag;
	}

	inline string GetObjectTag()
	{
		return _objectTag;
	}

	inline void PushCollisionData(string tag, collision_position position)
	{
		_collisionList.push_back(CollisionData{ tag, position });
	}


	inline bool CheckCollisionWith(string tag, collision_position position)
	{
		for (list<CollisionData>::iterator _collision = _collisionList.begin(); _collision != _collisionList.end(); _collision++)
		{
			if (_collision->collisionTag == tag && _collision->collisionPosition == position)
			{
				return true;
			}
		}

		return false;
	}

	inline bool CheckCollisionWith(string tag)
	{
		for (list<CollisionData>::iterator _collision = _collisionList.begin(); _collision != _collisionList.end(); _collision++)
		{
			if (_collision->collisionTag == tag)
			{
				return true;
			}
		}

		return false;
	}

	inline void ClearCollisionTags()
	{
		while (_collisionList.size()>0)
		{
			_collisionList.pop_back();
		}
	}

private:
	Vector2 m_position;
	Vector2 _potentialPosition;
	float _rotation;
	string _objectTag;
	list<CollisionData> _collisionList;
	Collider2D* m_collider;
	Rigidbody2D* m_rigidbody;
};

#endif