#ifndef _ANIMATED_SPRITE_H_
#define _ANIMATED_SPRITE_H_

#include <iostream>
#include <graphics/sprite.h>
#include <assets\png_loader.h>
#include <graphics\image_data.h>
#include <graphics\texture.h>
#include <system\platform.h>
#include <graphics\font.h>
#include <graphics\sprite_renderer.h>

class AnimatedSprite : public abfw::Sprite
{
public:
	AnimatedSprite()
	{
		anim_frame_count = 0;

		//Sets up a generic animation
		anim_speed = 1;
		anim_frame_swap = 30;
		num_frames = 8;
	}

	virtual ~AnimatedSprite();

public:
	virtual void Render(abfw::SpriteRenderer* sprite_renderer, abfw::Font& font);
	void LoadTexture(char* textureFilename, abfw::Platform& platform);
	
	inline void SetAnimFrameSwap(int frame_swap)
	{
		anim_frame_swap = frame_swap;

	}

	inline void SetAnimSpeed(int speed)
	{
		anim_speed = speed;
	}

	inline void SetFrameCount(int frames)
	{
		num_frames = frames;
	}

protected:
	void UpdateAnimation(float dt);
	void SwapAnimation(int anim_num);
	
public:
	bool visible_ = true;

protected:
	int anim_frame_swap;	
	int anim_frame_count;

	float anim_speed;
	float total_dt_ = 0.0f;

	int num_frames;
	int current_frame = 0;

	int num_cycles = 0;

};

#endif