#include "animation_object.h"

#include "sceneclass.h"

void AnimationObject::Frame(float dt)
{
	UpdateAnimation(dt);

	if (oneShot && num_cycles > 0)
		static_cast<SceneClass*>(owner_scene_)->delete_list_.push_back(this);
}

void AnimationObject::LoadUniqueData(char* buffer, int& bufferPosition)
{
	ParseBoolean(buffer, bufferPosition, oneShot);
	bufferPosition += 1;

	ParseInt(buffer, bufferPosition, num_frames);
	bufferPosition += 2;

	ParseFloat(buffer, bufferPosition, anim_speed);
	bufferPosition += 2;

	set_uv_width(1.0f / (float)num_frames);
	animation_time_ = (1.0f / anim_speed) * num_frames;
}

void AnimationObject::InstantiateCopy(const AnimationObject& other, float posX, float posY, float rot)
{
	platform_ = other.platform_;
	input_ = other.input_;
	audio_manager_ = other.audio_manager_;

	oneShot = other.oneShot;
	num_frames = other.num_frames;
	anim_speed = other.anim_speed;
	animation_time_ = other.animation_time_;

	owner_scene_ = other.owner_scene_;

	visible_ = true;

	set_uv_width(1.0f / (float)num_frames);

	int width = other.width_;
	int height = other.height_;

	float b2Width = ABFW_BOX2D_SIZE(other.width_) / 2;
	float b2Height = ABFW_BOX2D_SIZE(other.height_) / 2;

	colliderShape_ = other.colliderShape_;

	textureFilename_ = other.textureFilename_;

	if (strcmp(textureFilename_, "NULL") == 0)
	{
		visible_ = false;
	}
	else
	{
		set_texture(other.texture());
	}

	world_position_.x = posX;
	world_position_.y = posY;

	target_angle_ = rot;

	plat_height = (*platform_).height();

	set_width(width);
	set_height(height);

	InitializePhysics(other.object_body_->GetWorld(), rot, b2Width, b2Height, colliderShape_, other.hasCollision_, other.isKinematic_, other.fixedAngle_, other.density_, other.gravityScale_, other.friction_, other.restitution_);
}

float AnimationObject::GetAnimationTime()
{
	return animation_time_;
}