#include "audio_music.h"

void AudioMusic::Initialize(abfw::AudioManagerVita* audio_manager, abfw::Platform* platform, Trigger* trigger)
{
	sample_id_ = -1;
	sample_voice_ = -1;

	platform_ = platform;

	audio_manager_ = audio_manager;

	trigger_ = trigger;

	sample_volume_ = 1.0f;

	sample_pan_ = 0.0f;

	sample_pitch_ = 1.0f;

	playing_ = false;
}

void AudioMusic::PlaySound()
{
	if (sample_id_ != -1 && !playing_)
	{
		sample_voice_ = audio_manager_->PlayMusic();
		abfw::VolumeInfo volume_info;
		volume_info.volume = sample_volume_;
		volume_info.pan = sample_pan_;
		audio_manager_->SetSampleVoiceVolumeInfo(sample_voice_, volume_info);
		audio_manager_->SetSamplePitch(sample_voice_, sample_pitch_);
		playing_ = true;
	}
}

void AudioMusic::LoadSoundData(char* audioFilename)
{
	sample_id_ = audio_manager_->LoadMusic(audioFilename, *platform_);
}

void AudioMusic::Frame()
{
	if ((trigger_ == NULL || trigger_->Triggered()) && sample_id_ != -1 && !playing_)
	{
		sample_voice_ = audio_manager_->PlayMusic();
		abfw::VolumeInfo volume_info;
		volume_info.volume = sample_volume_;
		volume_info.pan = sample_pan_;
		audio_manager_->SetSampleVoiceVolumeInfo(sample_voice_, volume_info);
		audio_manager_->SetSamplePitch(sample_voice_, sample_pitch_);
		playing_ = true;
	}
}

void AudioMusic::Stop()
{
	if (sample_id_ != -1)
	{
		audio_manager_->StopMusic();
		playing_ = false;
	}
}

void AudioMusic::Shutdown()
{
	audio_manager_->UnloadMusic();
}