#include "audio_sfx.h"


void AudioSfx::Initialize(abfw::AudioManagerVita* audio_manager, abfw::Platform* platform, Trigger* trigger)
{
	sample_id_ = -1;
	sample_voice_ = -1;

	platform_ = platform;

	audio_manager_ = audio_manager;

	trigger_ = trigger;

	sample_volume_ = 1.0f;

	sample_pan_ = 0.0f;

	sample_pitch_ = 1.0f;

	looping_ = false;
}

void AudioSfx::LoadSoundData(char* audioFilename)
{
	sample_id_ = audio_manager_->LoadSample(audioFilename, *platform_);
}

void AudioSfx::Frame()
{
	bool triggered = trigger_->Triggered();

	if ((triggered || trigger_ == NULL) && sample_id_ != -1 && sample_voice_ == -1)
	{
		sample_voice_ = audio_manager_->PlaySample(sample_id_, looping_);
		abfw::VolumeInfo volume_info;
		volume_info.volume = sample_volume_;
		volume_info.pan = sample_pan_;
		audio_manager_->SetSampleVoiceVolumeInfo(sample_voice_, volume_info);
		audio_manager_->SetSamplePitch(sample_voice_, sample_pitch_);
	}
	else if (!triggered && sample_voice_ != -1 && looping_)
	{
		Stop();
	}
}


void AudioSfx::PlaySound()
{
	if (sample_id_ != -1 && (!looping_ || sample_voice_ != -1))
	{
		sample_voice_ = audio_manager_->PlaySample(sample_id_, looping_);
		abfw::VolumeInfo volume_info;
		volume_info.volume = sample_volume_;
		volume_info.pan = sample_pan_;
		audio_manager_->SetSampleVoiceVolumeInfo(sample_voice_, volume_info);
		audio_manager_->SetSamplePitch(sample_voice_, sample_pitch_);
	}
}

void AudioSfx::Stop()
{
	if (sample_voice_ != -1)
	{
		audio_manager_->StopPlayingSampleVoice(sample_voice_);
		sample_voice_ = -1;
	}
}

void AudioSfx::Shutdown()
{
	audio_manager_->UnloadSample(sample_id_);
}