#ifndef _BASE_SCENE_CLASS_
#define _BASE_SCENE_CLASS_

#include <system/file.h>
#include <system/platform.h>
#include <graphics/vita/sprite_renderer_vita.h>
#include <audio\vita\audio_manager_vita.h>
#include <list>


using abfw::File;
using abfw::Platform;
using std::list;
using std::iterator;

#include "ui_object.h"
#include "font_object.h"
#include "inputclass.h"
#include "game_object.h"
#include "options_storage.h"

class BaseSceneClass
{
protected:
	struct LoadStructure
	{
		float posX, posY;
		char* objectFilename;
		const char* type_tag;
	};

public:
	virtual ~BaseSceneClass()
	{
	}

	virtual bool Initialize(Platform* platform, abfw::AudioManagerVita* audio_manager, InputClass* input, abfw::SpriteRenderer* renderer, abfw::Font& font, OptionsStorage* options_storage, const char* filename) = 0;
	virtual int Frame(float dt) = 0;
	virtual void Shutdown() = 0;

	virtual void Render(abfw::SpriteRenderer* renderer_, abfw::Font& font);
	
	virtual void DeleteObject(GameObject* object);
	virtual GameObject* InstantiateObject(GameObject* object, float posX, float posY, float rot);


	abfw::AudioManagerVita* AudioManager()
	{
		return audio_manager_;
	}

protected:
	std::list<GameObject *> game_objects_;
	std::list<UIObject *> ui_objects_;
	InputClass* input_;
	Platform* platform_;
	AnimatedSprite* background_image_;
	abfw::AudioManagerVita* audio_manager_;
	OptionsStorage* options_storage_;

};

#endif