#include "box_trigger.h"

void BoxTrigger::Initialize(b2World* world, string target, float triggerX, float triggerY, float triggerWidth, float triggerHeight)
{
	world_ = world;
	target_tags_.push_back(target);

	b2BodyDef bodyDef;
	bodyDef.type = b2_staticBody;
	bodyDef.position = b2Vec2(triggerX, triggerY);
	trigger_body_ = world->CreateBody(&bodyDef);

	b2PolygonShape staticBox;
	staticBox.SetAsBox(triggerWidth, triggerHeight);

	b2FixtureDef fixtureDef;
	fixtureDef.shape = &staticBox;
	fixtureDef.isSensor = true;

	trigger_body_->CreateFixture(&fixtureDef);
}

void* BoxTrigger::Triggered()
{
	b2Contact* contact = NULL;
	
	if (trigger_body_->GetContactList())
		contact = trigger_body_->GetContactList()->contact;

	while (contact)
	{
		b2Body* bodyA = contact->GetFixtureA()->GetBody();
		b2Body* bodyB = contact->GetFixtureB()->GetBody();

		if (bodyA->GetUserData())
		{
			GameObject* game_object = (GameObject*)bodyA->GetUserData();
			string hit_tag = game_object->typeTag_;

			for (int i = 0; i < target_tags_.size(); i++)
			{
				if (hit_tag == target_tags_[i])
				{
					return bodyA->GetUserData();
				}
			}	
		}

		if (bodyB->GetUserData())
		{
			GameObject* game_object = (GameObject*)bodyB->GetUserData();
			string hit_tag = game_object->typeTag_;

			for (int i = 0; i < target_tags_.size(); i++)
			{
				if (hit_tag == target_tags_[i])
				{
					return bodyB->GetUserData();
				}
			}
		}

		contact = contact->GetNext();
	}

	return NULL;
}

void BoxTrigger::AddTarget(string new_target)
{
	target_tags_.push_back(new_target);
}