#include "enemy_object.h"

#include "sceneclass.h"

void EnemyObject::Frame(b2Vec2 player_position, float dt)
{
	UpdateAnimation(dt);

	if (weapon_)
	{
		b2Vec2 posit = player_position - object_body_->GetPosition();
		posit.Normalize();
		float angle = -atan2(posit.x, posit.y);
		posit *= (b2Width_ / 2);
		posit += object_body_->GetPosition();
		weapon_->Frame(posit, player_position, angle, dt);

		if ((player_position - posit).Length() <= attack_range_)
			weapon_->Fire();
	}


}

bool EnemyObject::TakeDamage(float damage_amount)
{
	if (health_ <= 0.0f)
		return true;

	health_ -= damage_amount;
	if (health_ < 0.0f)
		health_ = 0.0f;

	integer_health_ = (int)health_;

	if (health_ <= 0)
	{
		static_cast<SceneClass*>(owner_scene_)->InstantiateObject(death_animation_, world_position_.x, world_position_.y, 0.0f);
		static_cast<SceneClass*>(owner_scene_)->delete_list_.push_back(this);
		static_cast<SceneClass*>(owner_scene_)->enemy_count_--;
		if (static_cast<SceneClass*>(owner_scene_)->enemy_count_ <= 0)
		{
			static_cast<SceneClass*>(owner_scene_)->end_timer_ = GetDeathAnimationTime();
		}
	}

	return true;
}

void EnemyObject::Shutdown()
{
	weapon_->Shutdown();
	delete weapon_;
	weapon_ = NULL;
}

void EnemyObject::Render(abfw::SpriteRenderer* sprite_renderer, abfw::Font& font)
{
	if (visible_)
	{
		sprite_renderer->DrawSprite(*this);
		weapon_->Render(sprite_renderer, font);
	}
}

void EnemyObject::LoadUniqueData(char* buffer, int& bufferPosition)
{
	ParseInt(buffer, bufferPosition, max_health_);
	bufferPosition += 2;

	ParseCharArray(buffer, bufferPosition, weapon_filename_);
	bufferPosition += 2;

	ParseFloat(buffer, bufferPosition, attack_range_);
	bufferPosition += 2;

	ParseCharArray(buffer, bufferPosition, animation_filename_);
	bufferPosition += 2;

	health_ = max_health_;
	integer_health_ = (int)health_;

	b2Filter filter = object_body_->GetFixtureList()->GetFilterData();

	filter.categoryBits = 0x0008;

	object_body_->GetFixtureList()->SetFilterData(filter);

	set_uv_width(1.0f / 8.0f);

	anim_speed = 3.0f;
	num_frames = 8;

	if (strcmp(weapon_filename_, "NULL") != 0)
	{
		weapon_ = new WeaponObject;
		weapon_->LoadDataFile(weapon_filename_, platform_, object_body_->GetWorld(), audio_manager_, input_, options_storage_, 0.0f, 0.0f, 0.0f);

		weapon_->owner_scene_ = owner_scene_;
	}

	if (strcmp(animation_filename_, "NULL") != 0)
	{
		death_animation_ = new AnimationObject;
		death_animation_->LoadDataFile(animation_filename_, platform_, object_body_->GetWorld(), audio_manager_, input_, options_storage_, 0.0f, 0.0f, 0.0f);
		death_animation_->owner_scene_ = owner_scene_;
	}
}

float EnemyObject::GetDeathAnimationTime()
{
	if (death_animation_)
	{
		return death_animation_->GetAnimationTime();
	}
	else
	{
		return 0.0f;
	}
}