#include "forces.h"

void CalculateResultantForce(DirectForce& output, DirectForce force1, DirectForce force2)
{
	DirectForce tempForce( 0.0f, 0.0f , 0.0f, 0.0f);

	float betweenAngle;

	//Calculate the angle between the two inputted vectors in radians
	betweenAngle = acos(((force1.x_comp * force2.x_comp) + (force1.y_comp * force2.y_comp)) / (force1.magnitude / force2.magnitude));
	
	//Calculate the magnitude of the resultant vector
	tempForce.magnitude = sqrt(pow(force1.magnitude, 2) + pow(force2.magnitude, 2) + (2 * force1.magnitude*force2.magnitude*cos(betweenAngle)));

	//Calculate the angle of the resultant vector in degrees
	tempForce.angle = (atan((force2.magnitude * sin(betweenAngle)) / (force1.magnitude + (force2.magnitude * cos(betweenAngle)))) * 180 / F_PI);

	//Update the values in the output variable
	output.magnitude = tempForce.magnitude;
	output.angle = tempForce.angle;

	output.x_comp = (force1.x_comp + force2.x_comp);
	output.y_comp = (force1.y_comp + force2.y_comp);
}

float CalculateAngularAcceleration(DirectForce force, float inertia)
{
	float result;
	float tau;
	Vector2 component;
	Vector2 offset;

	offset = Vector2(force.x_offset, force.y_offset);
	component = Vector2(force.x_comp, force.y_comp);

	tau = offset.Cross(component);

	result = inertia * tau;

	return result;
}

DirectForce::DirectForce(float x_component, float y_component, float x_off, float y_off)
{
	magnitude = sqrt(pow(x_component, 2) + pow(y_component, 2));

	x_comp = x_component;
	y_comp = y_component;
	
	x_offset = x_off;
	y_offset = y_off;

	angle = CalculateAngleInCircle(x_component, y_component);
}