#include "game_object.h"

GameObject::GameObject()
{
	object_body_ = NULL;
	platform_ = NULL;
	set_width(32);
	set_height(32);
	visible_ = true;
}

GameObject::~GameObject()
{
}

bool GameObject::TakeDamage(float damage)
{
	return false;
}

void GameObject::InitGeneric(abfw::Platform* platform, b2World* physics_world, abfw::AudioManagerVita* audio_manager, InputClass* input, char* spriteFilename, float x, float y, float rot, float _width, float _height, int colliderShape, bool hasCollision, bool isKinematic, bool fixedRotation, float density, float gravityScale, float friction, float restitution)
{
	audio_manager_ = audio_manager;
	input_ = input;

	if (strcmp(spriteFilename, "NULL") == 0)
	{
		visible_ = false;
	}
	else
	{
		LoadTexture(spriteFilename, *platform);
	}

	set_colour(0xEEEEEEEE);

	world_position_.x = x;
	world_position_.y = y;
	
	target_angle_ = rot;

	plat_height = (*platform).height();

	set_width(BOX2D_ABFW_SIZE(_width));
	set_height(BOX2D_ABFW_SIZE(_height));
	
	InitializePhysics(physics_world, rot, _width, _height, colliderShape, hasCollision, isKinematic, fixedRotation, density, gravityScale, friction, restitution);
}

void GameObject::InitializePhysics(b2World* physics_world, float rot, float width, float height, int colliderShape, bool hasCollision, bool isKinematic, bool fixedRotation, float density, float gravityScale, float friction, float restitution)
{
	if (isKinematic)
	{
		b2BodyDef bodyDef;
		bodyDef.type = b2_dynamicBody;
		bodyDef.position = b2Vec2(world_position_.x, world_position_.y);
		bodyDef.fixedRotation = fixedRotation;
		object_body_ = physics_world->CreateBody(&bodyDef);
		object_body_->SetTransform(object_body_->GetPosition(), rot);
		object_body_->SetGravityScale(gravityScale);

		object_body_->SetUserData(this);

		b2FixtureDef fixtureDef;

		switch (colliderShape)
		{
			case 0:
			{
					  b2PolygonShape dynamicShape;
					  dynamicShape.SetAsBox(width / 2, height / 2);
					  fixtureDef.shape = &dynamicShape;
					  fixtureDef.density = density;
					  fixtureDef.friction = friction;
					  fixtureDef.restitution = restitution;
					  fixtureDef.filter.categoryBits = 0x0003;
					  if (!hasCollision)
						  fixtureDef.filter.maskBits = 0;

					  object_body_->CreateFixture(&fixtureDef);
					  break;
			}
			case 1:
			{
					  b2CircleShape circleShape;
					  circleShape.m_p.Set(0, 0);
					  circleShape.m_radius = width / 2;
					  fixtureDef.shape = &circleShape;
					  fixtureDef.density = density;
					  fixtureDef.friction = friction;
					  fixtureDef.restitution = restitution;
					  fixtureDef.filter.categoryBits = 0x0003;
					  if (!hasCollision)
						  fixtureDef.filter.maskBits = 0;

					  object_body_->CreateFixture(&fixtureDef);
					  break;
			}
		}		
	}
	else
	{
		b2BodyDef bodyDef;
		bodyDef.type = b2_staticBody;
		bodyDef.fixedRotation = fixedRotation;
		bodyDef.position = b2Vec2(world_position_.x, world_position_.y);
		object_body_ = physics_world->CreateBody(&bodyDef);
		object_body_->SetTransform(object_body_->GetPosition(), rot);

		object_body_->SetUserData(this);

		b2FixtureDef fixtureDef;

		switch (colliderShape)
		{
			case 0:
			{
					  b2PolygonShape dynamicShape;
					  dynamicShape.SetAsBox(width / 2, height / 2);
					  fixtureDef.shape = &dynamicShape;
					  fixtureDef.density = density;
					  fixtureDef.friction = friction;
					  fixtureDef.restitution = restitution;
					  fixtureDef.filter.categoryBits = 0x0002;
					  if (!hasCollision)
						  fixtureDef.filter.maskBits = 0;

					  object_body_->CreateFixture(&fixtureDef);
					  break;
			}
			case 1:
			{
					  b2CircleShape circleShape;
					  circleShape.m_p.Set(0, 0);
					  circleShape.m_radius = width / 2;
					  fixtureDef.shape = &circleShape;
					  fixtureDef.density = density;
					  fixtureDef.friction = friction;
					  fixtureDef.restitution = restitution;
					  fixtureDef.filter.categoryBits = 0x0002;
					  if (!hasCollision)
						  fixtureDef.filter.maskBits = 0;

					  object_body_->CreateFixture(&fixtureDef);
					  break;
			}
		}
	}

}

void GameObject::LoadDataFile(char* filename, abfw::Platform* platform, b2World* world_, abfw::AudioManagerVita* audio_manager, InputClass* _input, OptionsStorage* options_storage, float posX, float posY, float rot)
{
	int bufferPosition = 0;
	bool success;
	char input;
	float width, height;

	platform_ = platform;
	options_storage_ = options_storage;

	File* object_file = platform_->CreateFile();

	success = object_file->Open(filename);
	if (success)
	{
		char* buffer = NULL;

		Int32 file_size;
		success = object_file->GetSize(file_size);
		if (success)
		{
			buffer = new char[file_size +1];
			Int32 bytes_read;
			success = object_file->Read(buffer, file_size, bytes_read);
			buffer[file_size] = '\0';

			if (success)
				object_file->Close();

			if (success)
			{
				//Skip the first line as the data from that file has already been used
				input = buffer[bufferPosition];
				while (input != '\n')
				{
					bufferPosition++;
					input = buffer[bufferPosition];
				}
				bufferPosition++;
				
				//Read in the sprite filename
				ParseCharArray(buffer, bufferPosition, textureFilename_);
				bufferPosition+=2;

				//Read in the width
				ParseFloat(buffer, bufferPosition, width);
				b2Width_ = width;
				bufferPosition+=2;

				//Read in the height
				ParseFloat(buffer, bufferPosition, height);
				b2Height_ = height;	
				bufferPosition+=2;

				//Read in the collider shape
				ParseInt(buffer, bufferPosition, colliderShape_);
				bufferPosition += 2;

				//Read if the object has physics
				ParseBoolean(buffer, bufferPosition, hasCollision_);
				bufferPosition++;

				//Read if the object is kinematic (0/1)
				ParseBoolean(buffer, bufferPosition, isKinematic_);
				bufferPosition++;

				//Read if the object should have fixed rotation
				ParseBoolean(buffer, bufferPosition, fixedAngle_);
				bufferPosition++;
				
				//Read the density
				ParseFloat(buffer, bufferPosition, density_);
				bufferPosition+=2;

				//Read the gravity scale
				ParseFloat(buffer, bufferPosition, gravityScale_);
				bufferPosition+=2;

				//Read the friction
				ParseFloat(buffer, bufferPosition, friction_);
				bufferPosition+=2;

				//Read the restitution
				ParseFloat(buffer, bufferPosition, restitution_);
				bufferPosition+=2;

				InitGeneric(platform_, world_, audio_manager, _input, textureFilename_, posX, posY, rot, width, height, colliderShape_, hasCollision_, isKinematic_, fixedAngle_, density_, gravityScale_, friction_, restitution_);

				LoadUniqueData(buffer, bufferPosition);

				delete[] buffer;
				buffer = NULL;
			}
		}
	}

}

void GameObject::LoadUniqueData(char* buffer, int& bufferPosition)
{
}

void GameObject::FrameGeneric(b2Vec2 camera_position, float dt)
{
	b2Vec2 screen_position;

	world_position_ = object_body_->GetPosition();

	screen_position = world_position_ - camera_position;

	set_rotation(-(object_body_->GetAngle()));

	set_position(abfw::Vector3(BOX2D_ABFW_POS_X(screen_position.x), (BOX2D_ABFW_POS_Y(screen_position.y) - (plat_height/2)), 0.f));

	UpdateAnimation(dt);
}


void GameObject::Shutdown()
{

}

void GameObject::CreationFrame(b2Vec2 camera_position)
{
	b2Vec2 screen_position;

	set_width(width());
	set_height(height());

	screen_position = world_position_ - camera_position;

	set_position(BOX2D_ABFW_POS_X(screen_position.x), BOX2D_ABFW_POS_Y(screen_position.y), 0.f);

	UpdateAnimation(0.0f);
}

b2Vec2 GameObject::GetWorldPosition()
{
	return world_position_;
}

void GameObject::SetPosition(float x, float y)
{
	world_position_.x = x;
	world_position_.y = y;
}

abfw::Vector3 GameObject::GetPosition()
{
	return abfw::Vector3(world_position_.x, world_position_.y, 0.f);
}

void GameObject::SetTag(string tag)
{
	object_tag_ = tag;
}

string GameObject::GetTag()
{
	return object_tag_;
}
