#ifndef _GAME_OBJECT_H_
#define _GAME_OBJECT_H_

#include <iostream>
#include <string>
#include <math.h>

#include <system/file.h>
#include <graphics/sprite.h>
#include <maths/vector2.h>
#include <system/platform.h>
#include <audio\vita\audio_manager_vita.h>

#include <Box2D\Box2D.h>

#include "box2d_helpers.h"
#include "animated_sprite.h"
#include "BufferParser.h"
#include "font_object.h"
#include "inputclass.h"
#include "options_storage.h"

using std::string;
using abfw::File;

class GameObject : public AnimatedSprite
{
public:
	GameObject();
	virtual ~GameObject();

	void InitGeneric(abfw::Platform* platform, b2World* physics_world, abfw::AudioManagerVita* audio_manager, InputClass* input, char* spriteFilename, float x, float y, float rot, float width, float height, int colliderShape, bool hasCollision, bool isKinematic, bool fixedRotation, float density, float gravityScale, float friction, float restitution);
	void FrameGeneric(b2Vec2 camera_position, float dt);

	virtual bool TakeDamage(float damage_amount_);
	virtual void Shutdown();
	void CreationFrame(b2Vec2 camera_position);
	void SetPosition(float x, float y);
	void SetTag(string objectTag);
	void LoadDataFile(char* objectFilename, abfw::Platform* platform_, b2World* world, abfw::AudioManagerVita* audio_manager, InputClass* input, OptionsStorage* options_storage, float posX, float posY, float rot);
	
	string GetTag();

	abfw::Vector3 GetPosition();
	b2Vec2 GetWorldPosition();
	//abfw::Sprite GetSprite();

	b2Body* GetObjectBody()
	{
		return object_body_;
	}

protected:
	virtual void LoadUniqueData(char* buffer, int& bufferPosition);
	void InitializePhysics(b2World* physics_world, float rot, float width, float height, int colliderShape, bool hasCollision, bool isKinematic, bool fixedRotation, float density, float gravityScale, float friction, float restitution);

public:
	const char* typeTag_;
	void* owner_scene_;

protected:
	//abfw::Sprite _sprite;
	b2Vec2 world_position_;
	b2Vec2 world_rotation_;
	float target_angle_, rotation_threshold_;
	
	b2Body* object_body_;

	string object_tag_;

	float plat_height;

	abfw::Platform* platform_;
	abfw::AudioManagerVita* audio_manager_;
	InputClass* input_;
	OptionsStorage* options_storage_;

	bool hasCollision_, isKinematic_, fixedAngle_;
	float density_, gravityScale_, friction_, restitution_;
	float b2Width_, b2Height_;
	int colliderShape_;
	char* textureFilename_;
};

#endif