#ifndef _KINEMATIC_ANIMATION_H_
#define _KINEMATIC_ANIMATION_H_

#include <math.h>
#include <graphics\sprite.h>
#include <maths\vector2.h>

class KinematicAnimation
{
private:
	struct Bone
	{
		abfw::Vector2 start_position;
		abfw::Vector2 end_position;
		float bone_angle;
		float bone_length;
		abfw::Sprite bone_sprite;
	};

	struct Joint
	{
		abfw::Vector2 joint_position;
		abfw::Sprite joint_sprite;
	};

public:
	KinematicAnimation();
	KinematicAnimation(const KinematicAnimation& other);
	~KinematicAnimation();

	bool Initialize(abfw::Vector2 point, float bone_one_length, float bone_two_length);
	void Frame();
	void Shutdown();

	inline void SetStartPoint(abfw::Vector2 point)
	{
		start_joint.joint_position = point;
	};

	inline void SetStartPoint(float posX, float posY)
	{
		start_joint.joint_position = abfw::Vector2(posX, posY);
	};

	inline void SetEndPoint(abfw::Vector2 point)
	{
		end_joint.joint_position = point;
	};

	inline void SetEndPoint(float posX, float posY)
	{
		end_joint.joint_position = abfw::Vector2(posX, posY);
	};

private:
	void CalculateMidPoint();

private:
	Bone bone_one;
	Bone bone_two;
	Joint start_joint;
	Joint mid_joint;
	Joint end_joint;
};

#endif