#include "los_trigger.h"

void LOSTrigger::Initialize(b2World* world, string target_tag, b2Vec2 start_point, float view_angle, float view_rotation, float los_range)
{
	world_ = world;
	target_tag_ = target_tag;
	start_point_ = start_point;
	view_angle_ = view_angle;
	view_rotation_ = view_rotation;
	los_range_ = los_range;

	RecalculateViewCone();
}

void* LOSTrigger::Triggered()
{
	b2Vec2 objectVector;
	b2Body* objectBody;
	GameObject* gameObject;
	GameObject* closestObject;
	b2RayCastCallback* raycastCallback;
	b2Fixture* fixture;
	b2Fixture* tempFixture;

	objectBody = world_->GetBodyList();

	while (objectBody)
	{
		gameObject = (GameObject*)objectBody->GetUserData();

		if (gameObject->GetTag() == target_tag_)
		{
			objectVector = b2Vec2(gameObject->GetPosition().x, gameObject->GetPosition().y) - start_point_;

			//check if object is within the line of sight range
			if (objectVector.Length() < los_range_)
			{
				//Calculate if the object is within the view cone
				objectVector.Normalize();

				float dotA, dotB;

				//dotA = view_edges_[0].dot(objectVector);
				//dotB = view_edges_[1].dot(objectVector);

				if (dotA <0 && dotB > 0)
				{
					b2Vec2 end_point = objectVector;
					end_point *= los_range_;

					b2Vec2 hit_point;
					b2Vec2 hit_normal;

					//do the raycast
					world_->RayCast(raycastCallback, start_point_, end_point);

					float32 fraction = raycastCallback->ReportFixture(tempFixture, hit_point, hit_normal, 0);
					while (tempFixture)
					{
						fixture = tempFixture;
						end_point = objectVector;
						end_point *= (los_range_ * fraction);
						world_->RayCast(raycastCallback, start_point_, end_point);
						fraction = raycastCallback->ReportFixture(tempFixture, hit_point, hit_normal, 0);
					}

					if (((GameObject*)fixture->GetUserData())->GetTag() == target_tag_)
						return fixture->GetUserData();
				}
			}
			//Get the next body in the list
			objectBody = objectBody->GetNext();
		}
	}

	return NULL;
}

void LOSTrigger::RecalculateViewCone()
{
	b2Vec2 centreVector;
	float tri_angle;
	float x, y;

	switch ((int)floor(view_rotation_))
	{
	case 0:
		centreVector = b2Vec2(0.0f, 1.0f); break;
	case 90:
		centreVector = b2Vec2(1.0f, 0.0f); break;
	case 180:
		centreVector = b2Vec2(0.0f, -1.0f); break;
	case 270:
		centreVector = b2Vec2(-1.0f, 0.0f); break;
	default:
		tri_angle = view_rotation_ - (90 * (floor(view_rotation_ / 90)));
		float rad_angle = (tri_angle * b2_pi) / 180;
		x = 1 * cos(rad_angle);
		y = 1 * sin(rad_angle);
		if (view_rotation_ > 90 && view_rotation_ < 270)
			y = -y;
		if (view_rotation_ > 180 && view_rotation_ < 360)
			x = -x;
		centreVector = b2Vec2(x, y);
		break;
	}
	
	float rad_angle = ((view_angle_ * b2_pi) / 180)/2;

	view_edges_[0].x = (centreVector.x * cos(rad_angle)) + (centreVector.y * sin(rad_angle));
	view_edges_[0].y = (centreVector.x * -sin(rad_angle)) + (centreVector.y * cos(rad_angle));

	rad_angle = -rad_angle;

	view_edges_[1].x = (centreVector.x * cos(rad_angle)) + (centreVector.y * sin(rad_angle));
	view_edges_[1].y = (centreVector.x * -sin(rad_angle)) + (centreVector.y * cos(rad_angle));
}

void LOSTrigger::ChangeTarget(string tag)
{
	target_tag_ = tag;
}

void LOSTrigger::ChangeStartPoint(b2Vec2 start_point)
{
	start_point_ = start_point;
}

void LOSTrigger::ChangeViewAngle(float view_angle)
{
	view_angle_ = view_angle;

	RecalculateViewCone();
}

void LOSTrigger::ChangeViewRotation(float view_rotation)
{
	view_rotation_ = view_rotation;

	RecalculateViewCone();
}

void LOSTrigger::ChangeLosRange(float los_range)
{
	los_range_ = los_range;
}
