#include "options_scene.h"

bool OptionsScene::Initialize(Platform* platform, abfw::AudioManagerVita* audio_manager, InputClass* input, abfw::SpriteRenderer* renderer, abfw::Font& font, OptionsStorage* options_storage, const char* filename)
{
	platform_ = platform;

	input_ = input;

	audio_manager_ = audio_manager;

	options_storage_ = options_storage;

	volumeLeftButton_ = new UIButton;

	volumeLeftButton_->Initialize(*platform, "menu_arrow_left.png", input_, 330, 90, 150, 150, false);

	ui_objects_.push_back(volumeLeftButton_);

	volumeRightButton_ = new UIButton;

	volumeRightButton_->Initialize(*platform, "menu_arrow_right.png", input_, 630, 90, 150, 150, false);

	ui_objects_.push_back(volumeRightButton_);

	controlLeftButton_ = new UIButton;

	controlLeftButton_->Initialize(*platform, "menu_arrow_left.png", input_, 130, 440, 150, 150, false);

	ui_objects_.push_back(controlLeftButton_);

	controlRightButton_ = new UIButton;

	controlRightButton_->Initialize(*platform, "menu_arrow_right.png", input_, 830, 440, 150, 150, false);

	ui_objects_.push_back(controlRightButton_);

	difficultyLeftButton_ = new UIButton;

	difficultyLeftButton_->Initialize(*platform, "menu_arrow_left.png", input_, 330, 270, 150, 150, false);

	ui_objects_.push_back(difficultyLeftButton_);

	difficultyRightButton_ = new UIButton;

	difficultyRightButton_->Initialize(*platform, "menu_arrow_right.png", input_, 630, 270, 150, 150, false);

	ui_objects_.push_back(difficultyRightButton_);

	background_image_ = new AnimatedSprite;

	background_image_->LoadTexture("background.png", *platform);
	background_image_->set_width(1024);
	background_image_->set_height(1024);
	background_image_->set_position(abfw::Vector3(platform->width() / 2, platform->height() / 2, 0.9f));

	return true;
}

void OptionsScene::Shutdown()
{
	platform_ = NULL;
	input_ = NULL;
}


int	OptionsScene::Frame(float dt)
{
	float new_uv_x = background_image_->uv_position().x + (dt / 2.0f);

	if (new_uv_x >= 1.0f)
		new_uv_x -= 1.0f;

	background_image_->set_uv_position(abfw::Vector2(new_uv_x, 0.0f));

	if (volumeLeftButton_->Frame(dt))
	{
		if (options_storage_->GetMasterVolume() > 0.0f)
		{
			options_storage_->SetMasterVolume(options_storage_->GetMasterVolume() - 0.1f);

		}
	}

	if (volumeRightButton_->Frame(dt))
	{
		if (options_storage_->GetMasterVolume() < 1.0f)
		{
			options_storage_->SetMasterVolume(options_storage_->GetMasterVolume() + 0.1f);

		}
	}

	if (controlLeftButton_->Frame(dt))
	{
		if (current_control_ > 0)
		{
			current_control_ = (ControlEnum)((int)current_control_ - 1);
		}
	}

	if (controlRightButton_->Frame(dt))
	{
		if (current_control_ < 4)
		{
			current_control_ = (ControlEnum)((int)current_control_ + 1);
		}
	}

	if (difficultyLeftButton_->Frame(dt))
	{
		if (options_storage_->GetCurrentLevel()>1)
		{
			options_storage_->SetCurrentLevel(options_storage_->GetCurrentLevel() - 1);
		}
	}

	if (difficultyRightButton_->Frame(dt))
	{
		if (options_storage_->GetCurrentLevel()< options_storage_->GetLevelCount())
		{
			options_storage_->SetCurrentLevel(options_storage_->GetCurrentLevel() + 1);
		}
	}

	if (input_->IsAnyButtonPressed())
	{
		if (input_->ButtonPressed(ABFW_SONY_CTRL_START))
		{
			options_storage_->ChangeControlSetting(current_control_, input_->ButtonPressed(ABFW_SONY_CTRL_START));
		}
	}

	if (input_->IsButtonPressed(ABFW_SONY_CTRL_START))
		return returnValue_;

	return 3;
}

void OptionsScene::Render(abfw::SpriteRenderer* renderer_, abfw::Font& font)
{
	if (background_image_)
		background_image_->Render(renderer_, font);

	for (list<GameObject*>::iterator object = game_objects_.begin(); object != game_objects_.end(); object++)
	{
		if ((*object)->visible_)
			(*object)->Render(renderer_, font);
	}

	for (list<UIObject*>::iterator object = ui_objects_.begin(); object != ui_objects_.end(); object++)
	{
		if ((*object)->visible_)
			(*object)->Render(renderer_, font);
	}

	font.RenderText(renderer_, abfw::Vector3(480.0f, 0.0f, -0.9f), 6.0f, 0xff990099, abfw::TJ_CENTRE, "%d", (int)(options_storage_->GetMasterVolume()*10.0f));
	font.RenderText(renderer_, abfw::Vector3(480.0f, 180.0f, -0.9f), 6.0f, 0xff990099, abfw::TJ_CENTRE, "%d", options_storage_->GetCurrentLevel());

	font.RenderText(renderer_, abfw::Vector3(480.0f, 0.0f, -0.9f), 1.0f, 0xff990099, abfw::TJ_CENTRE, "Volume");
	font.RenderText(renderer_, abfw::Vector3(480.0f, 350.0f, -0.9f), 1.0f, 0xff990099, abfw::TJ_CENTRE, "Control Config");
	font.RenderText(renderer_, abfw::Vector3(480.0f, 180.0f, -0.9f), 1.0f, 0xff990099, abfw::TJ_CENTRE, "Current Level");

	switch (current_control_)
	{
	case BOOST:
		font.RenderText(renderer_, abfw::Vector3(480.0f, 350.0f, -0.9f), 6.0f, 0xff990099, abfw::TJ_CENTRE, "BOOST");
		break;
	case HOVER:
		font.RenderText(renderer_, abfw::Vector3(480.0f, 350.0f, -0.9f), 6.0f, 0xff990099, abfw::TJ_CENTRE, "HOVER");
		break;
	case FIRE:
		font.RenderText(renderer_, abfw::Vector3(480.0f, 350.0f, -0.9f), 6.0f, 0xff990099, abfw::TJ_CENTRE, "FIRE");
		break;
	case ALT_FIRE:
		font.RenderText(renderer_, abfw::Vector3(480.0f, 350.0f, -0.9f), 6.0f, 0xff990099, abfw::TJ_CENTRE, "ALT-FIRE");
		break;
	case EQUIPMENT_SWITCH:
		font.RenderText(renderer_, abfw::Vector3(480.0f, 350.0f, -0.9f), 6.0f, 0xff990099, abfw::TJ_CENTRE, "EQUIP");
		break;
	}
}