#include "pause_menu_scene.h"

bool PauseMenuSceneClass::Initialize(Platform* platform, abfw::AudioManagerVita* audio_manager, InputClass* input, abfw::SpriteRenderer* renderer, abfw::Font& font, OptionsStorage* options_storage, const char* filename)
{
	platform_ = platform;

	input_ = input;

	audio_manager_ = audio_manager;

	options_storage_ = options_storage;

	play_button_ = new UIButton;

	play_button_->Initialize(*platform, "play_button_anim.png", input_, 480, 100, 300, 150, true);

	ui_objects_.push_back(play_button_);

	options_button_ = new UIButton;

	options_button_->Initialize(*platform, "options_button_anim.png", input_, 480, 275, 300, 150, true);

	ui_objects_.push_back(options_button_);

	exit_button_ = new UIButton;

	exit_button_->Initialize(*platform, "exit_button_anim.png", input_, 480, 450, 300, 150, true);

	ui_objects_.push_back(exit_button_);

	background_image_ = new AnimatedSprite;

	background_image_->LoadTexture("background.png", *platform);
	background_image_->set_width(1024);
	background_image_->set_height(1024);
	background_image_->set_position(abfw::Vector3(platform->width() / 2, platform->height() / 2, 0.9f));

	return true;
}


void PauseMenuSceneClass::Shutdown()
{
	platform_ = NULL;
	input_ = NULL;
}

int PauseMenuSceneClass::Frame(float dt)
{
	float new_uv_x = background_image_->uv_position().x + (dt / 2.0f);

	if (new_uv_x >= 1.0f)
		new_uv_x -= 1.0f;

	background_image_->set_uv_position(abfw::Vector2(new_uv_x, 0.0f));

	if (exit_button_->Frame(dt))
		return 0;

	if (play_button_->Frame(dt))
		return 1;

	if (options_button_->Frame(dt))
		return 3;

	return 2;
}
