#include "player_object.h"

#include "sceneclass.h"

void PlayerObject::Frame(InputClass* input, float dt)
{
	float stick_x, stick_y;
	b2Contact* contact = NULL;
	float C = 2.0f;

	UpdateAnimation(dt);

	flame_sprites_[0].visible_ = false;
	flame_sprites_[1].visible_ = false;

	booster_sfx_.Frame();

	object_body_->SetLinearDamping(0.0f);

	if (weapon_)
	{
		b2Vec2 posit = b2Vec2(-sin(object_body_->GetAngle()) * ((b2Height_ / 2.0f) + 0.1f), cos(object_body_->GetAngle()) * ((b2Height_ / 2.0f) + 0.1f)) + object_body_->GetPosition();
		weapon_->Frame(posit, object_body_->GetPosition(), object_body_->GetAngle(), dt);
	}

	if (input->IsButtonPressed(options_storage_->GetControlSetting(BOOST)))
	{
		if (current_fuel_level_ > 0.0f)
		{
			input->LeftStickMagnitude(stick_x, stick_y);

			flame_sprites_[0].visible_ = true;
			flame_sprites_[1].visible_ = true;

			stick_y = -stick_y;

			flame_sprites_[0].set_uv_height(1.0f - (0.6f * (std::max(abs(stick_x), abs(stick_y)))));
			flame_sprites_[1].set_uv_height(1.0f - (0.6f * (std::max(abs(stick_x), abs(stick_y)))));

			current_fuel_level_ -= (fuel_use_rate_ * dt) * 0.5;

			object_body_->ApplyForceToCenter((object_body_->GetMass() * -(object_body_->GetWorld()->GetGravity())));

			if (object_body_->GetLinearVelocity().y < 0.0f)
			{
				object_body_->ApplyForceToCenter(b2Vec2(0.0f, (-object_body_->GetLinearVelocity().y)));
			}

			if (object_body_->GetLinearVelocity().y > 0.0f && stick_y == 0.0f && stick_x == 0.0f)
			{
				object_body_->SetLinearDamping(1.0f);
			}
			else if (object_body_->GetLinearVelocity().y > 0.0f)
			{
				object_body_->SetLinearVelocity(object_body_->GetLinearVelocity());
			}

			object_body_->ApplyForceToCenter(b2Vec2(stick_x, stick_y));

			if (stick_x != 0 || stick_y != 0)
				current_fuel_level_ -= (fuel_use_rate_ * dt) * 0.5;

			fuel_level_ = (int)current_fuel_level_;

			double relativeAngle = object_body_->GetAngle() - target_angle_;
			if (abs(relativeAngle) >= rotation_threshold_)
			{
				object_body_->SetAngularDamping(1.0f);
				object_body_->ApplyTorque(-relativeAngle * C);
			}
		}
	}

	if (input->IsButtonPressed(options_storage_->GetControlSetting(HOVER)))
	{
		//object_body_->ApplyForceToCenter(object_body_->GetMass() * -(object_body_->GetWorld()->GetGravity()) + -(object_body_->GetLinearVelocity()));
	}

	if (input->IsButtonPressed(options_storage_->GetControlSetting(FIRE)))
	{
		weapon_->Fire();
	}

	if (input->IsButtonPressed(options_storage_->GetControlSetting(ALT_FIRE)))
	{

	}

	if (input->IsButtonPressed(options_storage_->GetControlSetting(EQUIPMENT_SWITCH)))
	{

	}

	if (object_body_->GetContactList())
		contact = object_body_->GetContactList()->contact;

	while (contact)
	{
		b2WorldManifold worldManifold;
		contact->GetWorldManifold(&worldManifold);

		b2Body* bodyA = contact->GetFixtureA()->GetBody();
		b2Body* bodyB = contact->GetFixtureB()->GetBody();
		if (bodyA->GetUserData())
		{
			GameObject* game_object = (GameObject*)bodyA->GetUserData();
			string hit_tag = game_object->typeTag_;

			if (hit_tag == "TerrainObject" && worldManifold.points[0].y < bodyB->GetPosition().y)
			{
				current_fuel_level_ += (fuel_refresh_rate_ * dt);
				if (current_fuel_level_ > max_fuel_level_)
					current_fuel_level_ = max_fuel_level_;
				fuel_level_ = (int)current_fuel_level_;
				break;
			}
		}

		if (bodyB->GetUserData())
		{
			GameObject* game_object = (GameObject*)bodyB->GetUserData();
			string hit_tag = game_object->typeTag_;

			if (hit_tag == "TerrainObject" && worldManifold.points[0].y < bodyA->GetPosition().y)
			{
				current_fuel_level_ += (fuel_refresh_rate_ * dt);
				if (current_fuel_level_ > max_fuel_level_)
					current_fuel_level_ = max_fuel_level_;
				fuel_level_ = current_fuel_level_;
				break;
			}
		}

		contact = contact->GetNext();
	}
	
	AlignFlames();
}


void PlayerObject::AlignFlames()
{
	if (abs(rotation_) > 0.0f)
	{
		float angle;
		float angle_r;
		float x_offset, y_offset;
		b2Vec2 vectorA, vectorNormal;
		float rotation = rotation_ * (180 / b2_pi);

		if (rotation < 0.0f)
			rotation = 360.0f - rotation;

		float hypotenuse = (height_ / 2.0f) + 12.0f;
		
		angle = rotation - (90 * ((int)rotation / 90));

		angle_r = angle * (b2_pi / 180);

		if (rotation < 180)
		{
			if (rotation < 90)
			{
				x_offset = -(sin(angle_r)*hypotenuse);
				y_offset = cos(angle_r)*hypotenuse;
			}
			else
			{
				x_offset = -(cos(angle_r)*hypotenuse);
				y_offset = -(sin(angle_r)*hypotenuse);

			}
		}
		else
		{
			if (rotation < 270)
			{
				x_offset = sin(angle_r)*hypotenuse;
				y_offset = -(cos(angle_r)*hypotenuse);
			}
			else
			{
				x_offset = sin(angle_r)*hypotenuse;
				y_offset = cos(angle_r)*hypotenuse;

			}
		}

		vectorA = b2Vec2(x_offset, y_offset);
		vectorA.Normalize();

		vectorNormal = b2Vec2(-vectorA.y, vectorA.x);
		
		b2Vec2 normalA = vectorNormal;
		normalA *= -16;
		b2Vec2 normalB = vectorNormal;
		normalB *= 16;

		b2Vec2 offset = b2Vec2(x_offset, y_offset) + (normalA);
		flame_sprites_[0].set_position(abfw::Vector3(position_.x + offset.x, position_.y + offset.y, 0.0f));
		offset = b2Vec2(x_offset, y_offset) + (normalB);
		flame_sprites_[1].set_position(abfw::Vector3(position_.x + offset.x, position_.y + offset.y, 0.0f));

	}
	else
	{
		flame_sprites_[0].set_position(abfw::Vector3(position_.x - 16, position_.y + (height_ / 2) + 12, 0.0f));
		flame_sprites_[1].set_position(abfw::Vector3(position_.x + 16, position_.y + (height_ / 2) + 12, 0.0f));
	}
	flame_sprites_[0].set_rotation(rotation_);
	flame_sprites_[1].set_rotation(rotation_);
}

void PlayerObject::Render(abfw::SpriteRenderer* sprite_renderer, abfw::Font& font)
{
	if (visible_)
	{
		sprite_renderer->DrawSprite(*this);
		flame_sprites_[0].Render(sprite_renderer, font);
		flame_sprites_[1].Render(sprite_renderer, font);
		weapon_->Render(sprite_renderer, font);
	}
}

bool PlayerObject::TakeDamage(float damage_amount_)
{
	if (health_ <= 0.0f)
		return true;

	health_ -= damage_amount_;
	if (health_ < 0)
		health_ = 0;

	integer_health_ = (int)health_;

	if (health_ <= 0)
	{
		static_cast<SceneClass*>(owner_scene_)->InstantiateObject(death_animation_, world_position_.x, world_position_.y, 0.0f);
		static_cast<SceneClass*>(owner_scene_)->delete_list_.push_back(this);
	}

	return true;
}

void PlayerObject::LoadUniqueData(char* buffer, int& bufferPosition)
{
	char* booster_filename;

	//Read in the player max health value
	ParseInt(buffer, bufferPosition, player_max_health_);
	bufferPosition += 2;

	ParseInt(buffer, bufferPosition, max_fuel_level_);
	bufferPosition += 2;

	ParseFloat(buffer, bufferPosition, fuel_use_rate_);
	bufferPosition += 2;

	ParseFloat(buffer, bufferPosition, fuel_refresh_rate_);
	bufferPosition += 2;

	ParseCharArray(buffer, bufferPosition, booster_filename);
	bufferPosition += 2;

	ParseCharArray(buffer, bufferPosition, weapon_filename_);
	bufferPosition += 2;

	ParseCharArray(buffer, bufferPosition, animation_filename_);
	bufferPosition += 2;

	b2Filter filter = object_body_->GetFixtureList()->GetFilterData();

	filter.categoryBits = 0x0004;
	
	object_body_->GetFixtureList()->SetFilterData(filter);

	current_fuel_level_ = max_fuel_level_;
	fuel_level_ = max_fuel_level_;

	health_ = player_max_health_;
	integer_health_ = (int)health_;

	set_uv_width(1.0f / 8.0f);
	
	anim_speed = 3.0f;
	num_frames = 8;

	rotation_threshold_ = 5.0f * (b2_pi/180.0f);

	for (int i = 0; i < 2; i++)
	{
		flame_sprites_[i].LoadTexture(booster_filename, *platform_);
		flame_sprites_[i].set_width(24);
		flame_sprites_[i].set_height(24);
	}

	ButtonTrigger* trigger = new ButtonTrigger;
	trigger->Initialize(input_, options_storage_->GetControlSetting(BOOST));

	booster_sfx_.Initialize(audio_manager_, platform_, trigger);

	booster_sfx_.LoadSoundData("SpaceShip_Engine_Large_Loop_00.wav");

	booster_sfx_.ToggleLooping();

	booster_sfx_.SetSampleVolume(0.75f);

	if (strcmp(weapon_filename_, "NULL") != 0)
	{
		weapon_ = new WeaponObject;
		weapon_->LoadDataFile(weapon_filename_, platform_, object_body_->GetWorld(), audio_manager_, input_, options_storage_, 0.0f, 0.0f, 0.0f);
		
		weapon_->owner_scene_ = owner_scene_;
	}

	if (strcmp(animation_filename_, "NULL") != 0)
	{
		death_animation_ = new AnimationObject;
		death_animation_->LoadDataFile(animation_filename_, platform_, object_body_->GetWorld(), audio_manager_, input_, options_storage_, 0.0f, 0.0f, 0.0f);
		death_animation_->owner_scene_ = owner_scene_;
	}
}

float PlayerObject::GetDeathAnimationTime()
{
	if (death_animation_)
	{
		return death_animation_->GetAnimationTime();
	}
	else
	{
		return 0.0f;
	}
}

void PlayerObject::Shutdown()
{
	weapon_->Shutdown();
	delete weapon_;
	weapon_ = NULL;

	audio_manager_->UnloadSample(booster_sfx_.GetId());
}