#ifndef _PLAYER_OBJECT_H_
#define _PLAYER_OBJECT_H_

#include <algorithm>

#include "game_object.h"
#include "equipment_object.h"
#include "inputclass.h"
#include "audio_sfx.h"
#include "weapon_object.h"
#include "animation_object.h"


class PlayerObject : public GameObject
{
public:
	PlayerObject()
	{
		typeTag_ = "PlayerObject";
	}
	void Frame(InputClass* input, float dt);
	void Render(abfw::SpriteRenderer* sprite_renderer, abfw::Font& font);
	void Shutdown();
	void LoadUniqueData(char* buffer, int& bufferPosition);
	bool TakeDamage(float damage_amount_);
	float GetDeathAnimationTime();

	inline int* GetHealthPointer()
	{
		return &integer_health_;
	}

	inline int* GetMaxHealthPointer()
	{
		return &player_max_health_;
	}

	inline int* GetFuelPointer()
	{
		return &fuel_level_;
	}

	inline int* GetMaxFuelPointer()
	{
		return &max_fuel_level_;
	}

private:
	void ChangeEquipment();
	void AlignFlames();
	void AlignWeapon();

public:

private:
	int integer_health_;
	float health_;
	int player_max_health_;
	float equipment_rotation_;
	float fuel_use_rate_;
	bool landing_extended_;
	float current_fuel_level_;
	float fuel_refresh_rate_;
	int	fuel_level_;
	int max_fuel_level_;
	AnimatedSprite flame_sprites_[2];
	float left_length_, right_length_;
	AudioSfx booster_sfx_;
	char* weapon_filename_;
	WeaponObject* weapon_;
	char* animation_filename_;
	AnimationObject* death_animation_;

};

#endif