#include "pong_objects.h"

void Paddle::Initialize(float platform_height, float platform_width, b2World* world)
{
	set_height(128);
	set_width(32);

	if (player_num_ == 1)
	{
		set_position(224.0f, platform_height * 0.5f, 0.f);
	}
	else
	{
		set_position(platform_width - 224.0f, platform_height *0.5f, 0.f);
	}

	b2BodyDef bodyDef;
	bodyDef.type = b2_dynamicBody;
	bodyDef.position = b2Vec2(ABFW_BOX2D_POS_X(position_.x), ABFW_BOX2D_POS_Y(position_.y));
	object_body_ = world->CreateBody(&bodyDef);
	object_body_->SetGravityScale(0);

	object_body_->SetUserData(this);

	b2PolygonShape dynamicBox;
	dynamicBox.SetAsBox(ABFW_BOX2D_SIZE(width_ /2), ABFW_BOX2D_SIZE(height_));

	b2FixtureDef fixtureDef;
	fixtureDef.shape = &dynamicBox;
	fixtureDef.density = 1.0f;
	fixtureDef.friction = 0.1f;
	fixtureDef.restitution = 0.8f;
	object_body_->CreateFixture(&fixtureDef);
}

void Paddle::Frame(InputClass* input)
{
	float stickX, stickY;
	
	if (player_num_ == 1)
	{
		input->LeftStickMagnitude(stickX, stickY);
		object_body_->ApplyForceToCenter(b2Vec2(stickX, -stickY));
	}
	else
	{
		input->RightStickMagnitude(stickX, stickY);
		object_body_->ApplyForceToCenter(b2Vec2(stickX, -stickY));
	}

	set_position(abfw::Vector3(BOX2D_ABFW_POS_X(object_body_->GetPosition().x), BOX2D_ABFW_POS_Y(object_body_->GetPosition().y), 0.f));

}

void Ball::Initialize(float platform_height, float platform_width, b2World* world)
{
	set_height(32);
	set_width(32);

	set_position(abfw::Vector3(platform_width*0.5f + 128, platform_height*0.5f, 0.f));

	b2BodyDef bodyDef;
	bodyDef.type = b2_dynamicBody;
	bodyDef.position = b2Vec2(ABFW_BOX2D_POS_X(position_.x), ABFW_BOX2D_POS_Y(position_.y));
	object_body_ = world->CreateBody(&bodyDef);

	object_body_->SetUserData(this);

	b2PolygonShape dynamicBox;
	dynamicBox.SetAsBox(ABFW_BOX2D_SIZE(width_ / 2), ABFW_BOX2D_SIZE(height_ / 2));

	b2FixtureDef fixtureDef;
	fixtureDef.shape = &dynamicBox;
	fixtureDef.density = 0.3f;
	fixtureDef.friction = 0.1f;
	fixtureDef.restitution = 1.0f;
	object_body_->CreateFixture(&fixtureDef);
}

void Ball::Frame(InputClass* input)
{
	set_position(abfw::Vector3(BOX2D_ABFW_POS_X(object_body_->GetPosition().x), BOX2D_ABFW_POS_Y(object_body_->GetPosition().y), 0.f));
}

void Table::Initialize(float platform_height, float platform_width, b2World* world)
{
	set_height(32);
	set_width(platform_width * 0.75);

	set_position(abfw::Vector3(platform_width*0.5f, platform_height - 128, 0.f));

	b2BodyDef bodyDef;
	bodyDef.type = b2_staticBody;
	bodyDef.position = b2Vec2(ABFW_BOX2D_POS_X(position_.x), ABFW_BOX2D_POS_Y(position_.y));
	object_body_ = world->CreateBody(&bodyDef);

	object_body_->SetUserData(this);

	b2PolygonShape staticBox;
	staticBox.SetAsBox(ABFW_BOX2D_SIZE(width_ / 2), ABFW_BOX2D_SIZE(height_ / 2));

	object_body_->CreateFixture(&staticBox, 0.0f);
}

void Table::Frame(InputClass* input)
{
	set_position(abfw::Vector3(BOX2D_ABFW_POS_X(object_body_->GetPosition().x), BOX2D_ABFW_POS_Y(object_body_->GetPosition().y), 0.f));
}