#include "projectile_object.h"

#include "sceneclass.h"

void ProjectileObject::InstantiateCopy(const ProjectileObject& other, float posX, float posY, float rot)
{
	platform_ = other.platform_;
	input_ = other.input_;
	audio_manager_ = other.audio_manager_;

	flatDamage_ = other.flatDamage_;
	damage_amount_ = other.damage_amount_;
	thrust_amount_ = other.thrust_amount_;
	exist_time_ = other.exist_time_;
	target_filter_ = other.target_filter_;

	int width = other.width_;
	int height = other.height_;

	colliderShape_ = other.colliderShape_;

	float b2Width = ABFW_BOX2D_SIZE(other.width_)/2;
	float b2Height = ABFW_BOX2D_SIZE(other.height_) / 2;

	textureFilename_ = other.textureFilename_;

	if (strcmp(textureFilename_, "NULL")==0)
	{
		visible_ = false;
	}
	else
	{
		set_texture(other.texture());
	}

	world_position_.x = posX;
	world_position_.y = posY;

	target_angle_ = rot;

	plat_height = (*platform_).height();

	set_width(width);
	set_height(height);

	InitializePhysics(other.object_body_->GetWorld() , rot, b2Width, b2Height, colliderShape_, other.hasCollision_, other.isKinematic_, other.fixedAngle_, other.density_, other.gravityScale_, other.friction_, other.restitution_);

	SetTargetFilter(target_filter_);
}

void ProjectileObject::Frame(float dt)
{
	total_dt_ += dt;

	b2Contact* contact = NULL;

	b2Vec2 direction = b2Vec2(cos(object_body_->GetAngle()), sin(object_body_->GetAngle()));
	direction *= thrust_amount_;

	object_body_->ApplyLinearImpulse(direction, b2Vec2(0, 0));

	if (object_body_->GetContactList())
		contact = object_body_->GetContactList()->contact;

	if (contact)
	{
		float damage;
		GameObject* bodyA = (GameObject*)contact->GetFixtureA()->GetBody()->GetUserData();
		GameObject* bodyB = (GameObject*)contact->GetFixtureB()->GetBody()->GetUserData();

		if (!flatDamage_)
		{
			damage = damage_amount_ * (object_body_->GetLinearVelocity().Length());
		}
		else
		{
			damage = damage_amount_;
		}

		if (bodyA != this && bodyA)
		{
			bodyA->TakeDamage(damage);
			static_cast<SceneClass*>(owner_scene_)->delete_list_.push_back(this);
		}
		else if (bodyB != this && bodyB)
		{
			bodyB->TakeDamage(damage);
			static_cast<SceneClass*>(owner_scene_)->delete_list_.push_back(this);
		}
	}
	
	if (total_dt_ >= exist_time_)
		static_cast<SceneClass*>(owner_scene_)->delete_list_.push_back(this);
}

void ProjectileObject::LoadUniqueData(char* buffer, int& bufferPosition)
{
	ParseBoolean(buffer, bufferPosition, flatDamage_);
	bufferPosition += 1;

	ParseFloat(buffer, bufferPosition, damage_amount_);
	bufferPosition += 2;

	ParseFloat(buffer, bufferPosition, thrust_amount_);
	bufferPosition += 2;

	ParseFloat(buffer, bufferPosition, exist_time_);
	bufferPosition += 2;
}

void ProjectileObject::SetTargetFilter(uint16 target_filter)
{
	target_filter_ = target_filter;

	b2Filter filter = object_body_->GetFixtureList()->GetFilterData();

	filter.maskBits = 0x0002 | target_filter_;
	if (target_filter_ == 4)
	{
		filter.categoryBits = 0x0008;
	}
	else
	{
		filter.categoryBits = 0x0004;
	}

	object_body_->GetFixtureList()->SetFilterData(filter);
}