#ifndef _RIGIDBODY_H_
#define _RIGIDBODY_H_

#include <list>

using std::list;

#include "forces.h"
#include "vector2.h"

class Rigidbody2D
{
public:
	Rigidbody2D();
	Rigidbody2D(const Rigidbody2D&);
	~Rigidbody2D();

	void Initialize(float mass, Vector2 center, float area);

	inline void SetMass(float mass)
	{
		_mass = mass;
	}

	inline float GetMass()
	{
		return _mass;
	}

	inline float GetDensity()
	{
		return _density;
	}

	inline void SetVelocity(Vector2 velocity)
	{
		_velocity = velocity;
	}

	inline Vector2 GetVelocity()
	{
		return _velocity;
	}

	inline void ApplyForce(DirectForce force)
	{
		_actingForces.push_back(force);
	}

	inline void PopFrontForce()
	{
		_actingForces.pop_front();
	}

	inline int GetNumForces()
	{
		return _actingForces.size();
	}

	inline DirectForce ReturnForce()
	{
		DirectForce returnForce = _actingForces.front();
		_actingForces.pop_front();
		return returnForce;
	}

	inline void SetAcceleration(Vector2 accel)
	{
		_acceleration = accel;
	}

	inline Vector2 GetAcceleration()
	{
		return _acceleration;
	}

	inline void SetAngularVelocity(float av)
	{
		_angularVelocity = av;
	}

	inline float GetAngularVelocity()
	{
		return _angularVelocity;
	}

	inline void SetInertiaTensor()
	{
		_inertiaTensor = 1.0f - _mass * _massCenter.Dot(_massCenter);
		_inverseInertia = 1.0f / _inertiaTensor;
	}

	inline float GetInertiaTensor()
	{
		return _inertiaTensor;
	}

	inline float GetInverseInertia()
	{
		return _inverseInertia;
	}

private:
	float _mass;
	float _density;
	float _restitution; //value between 0.0f and 1.0f - unimplemented
	float _angularVelocity;
	float _inertiaTensor;
	float _inverseInertia;
	Vector2 _velocity;
	Vector2 _acceleration;
	Vector2 _massCenter;
	list<DirectForce> _actingForces;
};
#endif