#include "scene_creator_class.h"

SceneCreatorClass::SceneCreatorClass()
{
	level_count_ = 0;
	current_position = abfw::Vector2(16, 16);
	deltaTime = 0.0f;
	player_placed = false;
	goal_placed = false;
	wall_placed = false;
	input_ = NULL;
	platform_ = NULL;
}

SceneCreatorClass::SceneCreatorClass(const SceneCreatorClass& other)
{
}

SceneCreatorClass::~SceneCreatorClass()
{
}

bool SceneCreatorClass::Initialize(Platform& platform, InputClass* input, const char* filename)
{
	bool result;

	platform_ = &platform;

	result = LoadLevelList();

	if (!result)
	{
		return false;
	}

	platform_width_ = platform_->width();
	platform_height_ = platform_->height();
	current_position = abfw::Vector2(platform_->width() / 2, platform_->height() / 2);
	input_ = input;
	ChangeMode(placement_state::WALL);

	return true;
}

void SceneCreatorClass::Shutdown()
{
	while (game_objects.size() > 0)
	{
		game_objects.back()->Shutdown();
		delete game_objects.back();
		game_objects.pop_back();
	}
}

bool SceneCreatorClass::LoadLevelList()
{
	int bufferPosition;
	int tempBufferPosition;
	char input;
	int stringLength;
	char* charInput;
	int i;
	int hundreds, tens, ones;
	File* level_file = platform_->CreateFile();

	bool success = level_file->Open("level_list.txt");
	if (success)
	{
		while (filenames_.size()>0)
		{
			filenames_.pop_back();
		}

		char* buffer = NULL;
		Int32 file_size;
		success = level_file->GetSize(file_size);
		if (success)
		{
			buffer = new char[file_size];
			Int32 bytes_read;
			success = level_file->Read(buffer, file_size, bytes_read);

			if (success)
				level_file->Close();

			if (success)
			{
				//Read the level count
				bufferPosition = 0;
				hundreds = buffer[bufferPosition] - '0';
				bufferPosition++;
				tens = buffer[bufferPosition] - '0';
				bufferPosition++;
				ones = buffer[bufferPosition] - '0';

				level_count_ = (hundreds * 100) + (tens * 10) + ones;

				//Read in all the filenames
				for (int x = 0; x < level_count_; x++)
				{
					bufferPosition++;
					//Move to the filename
					input = buffer[bufferPosition];
					while (input < 48)
					{
						bufferPosition++;
						input= buffer[bufferPosition];
					}

					tempBufferPosition = bufferPosition;
					stringLength = 0;
					while (input  > 47)
					{
						tempBufferPosition++;
						stringLength++;
						input = buffer[tempBufferPosition];
						if (input == '.')
						{
							stringLength++;
							tempBufferPosition++;
							input = buffer[bufferPosition];
						}
					}
					charInput = new char[stringLength+1];
					i = 0;
					input = buffer[bufferPosition];
					while (input >47)
					{
						charInput[i] = input;
						i++;
						bufferPosition++;
						input = buffer[bufferPosition];
						if (input == '.')
						{
							charInput[i] = input;
							bufferPosition++;
							i++;
							input = buffer[bufferPosition];
						}
					}
					//Add a string terminator to the end of the filename
					charInput[stringLength] = '\0';
					string stringInput(charInput);
					filenames_.push_back(stringInput);
				}

				return true;
			}
		}
	}

	return false;
}

void SceneCreatorClass::Render(abfw::SpriteRendererVita* renderer_, abfw::Font& font)
{
	if (game_objects.size() > 0)
	{
		for (list<GameObject*>::iterator object = game_objects.begin(); object != game_objects.end(); object++)
		{
		renderer_->DrawSprite(*(*object));
		}
	}

	float frame_rate = 1.0f / dt_;
	char fps[20];
	std::snprintf(fps, 20, "FPS: %.1f", frame_rate);

	font.RenderText(renderer_, abfw::Vector3(10.0f, 5.0f, -0.9f), 1.0f, 0xff0000ff, abfw::TJ_LEFT, fps);
}

int SceneCreatorClass::Frame(float dt)
{
	deltaTime += dt;

	dt_ = dt;

	if (deltaTime >= 0.5f)
	{
		//Handle certain button inputs differently based on the current type of object
		switch (current_mode)
		{
		case(placement_state::WALL) :
			if (input_->IsButtonPressed(ABFW_SONY_CTRL_CROSS))
			{
				GameObject* objectPtr = NULL;

				objectPtr = new GameObject;

				objectPtr->InitGeneric(*platform_, current_position.x, current_position.y, 32, 32, "wall", 0xffffffff, platform_height_);

				game_objects.push_back(objectPtr);

				wall_placed = true;

				break;
			}
			else if (input_->IsButtonPressed(ABFW_SONY_CTRL_CIRCLE))
			{
				if (!player_placed)
				{
					ChangeMode(placement_state::PLAYER);
					break;
				}

				if (!goal_placed)
				{
					ChangeMode(placement_state::GOAL);
					break;
				}
			}
			else if (input_->IsButtonPressed(ABFW_SONY_CTRL_SQUARE))
			{
				ChangeMode(placement_state::ZONE);
				break;
			}

			break;

		case(placement_state::ZONE) :
			if (input_->IsButtonPressed(ABFW_SONY_CTRL_CROSS))
			{
				ChangeMode(placement_state::WALL);
				break;
			}
			else if (input_->IsButtonPressed(ABFW_SONY_CTRL_CIRCLE))
			{
				if (!player_placed)
				{
					ChangeMode(placement_state::PLAYER);
					break;
				}

				if (!goal_placed)
				{
					ChangeMode(placement_state::GOAL);
					break;
				}
			}
			else if (input_->IsButtonPressed(ABFW_SONY_CTRL_SQUARE))
			{
				GameObject* objectPtr = NULL;

				objectPtr = new GameObject;

				objectPtr->Initialize(current_position.x, current_position.y, 32, 32, "zone", 0xff0000ff, platform_height_);

				game_objects.push_back(objectPtr);

				break;
			}

			break;

		case(placement_state::PLAYER) :
			if (input_->IsButtonPressed(ABFW_SONY_CTRL_CROSS))
			{
				ChangeMode(placement_state::WALL);
				break;
			}
			else if (input_->IsButtonPressed(ABFW_SONY_CTRL_CIRCLE))
			{
				player_placed = true;
				ChangeModeNoDelete(placement_state::GOAL);
				break;
			}
			else if (input_->IsButtonPressed(ABFW_SONY_CTRL_SQUARE))
			{
				ChangeMode(placement_state::ZONE);
				break;
			}

			break;

		case(placement_state::GOAL) :
			if (input_->IsButtonPressed(ABFW_SONY_CTRL_CROSS))
			{
				ChangeMode(placement_state::WALL);
				break;
			}
			else if (input_->IsButtonPressed(ABFW_SONY_CTRL_CIRCLE))
			{
				goal_placed = true;
				ChangeModeNoDelete(placement_state::WALL);
				break;
			}
			else if (input_->IsButtonPressed(ABFW_SONY_CTRL_SQUARE))
			{
				ChangeMode(placement_state::ZONE);
				break;
			}

			break;

		case(placement_state::NOMODE) :
			if (input_->IsButtonPressed(ABFW_SONY_CTRL_CROSS))
			{
				ChangeModeNoDelete(placement_state::WALL);
				break;
			}
			else if (input_->IsButtonPressed(ABFW_SONY_CTRL_CIRCLE))
			{
				if (!player_placed)
				{
					ChangeModeNoDelete(placement_state::PLAYER);
					break;
				}

				if (!goal_placed)
				{
					ChangeModeNoDelete(placement_state::GOAL);
					break;
				}
			}
			else if (input_->IsButtonPressed(ABFW_SONY_CTRL_SQUARE))
			{
				ChangeModeNoDelete(placement_state::ZONE);
				break;
			}

			break;
		}

		//Generic button presses
		if (input_->IsButtonPressed(ABFW_SONY_CTRL_TRIANGLE))
		{
			if (game_objects.size() > 0)
			{
				ChangeMode(placement_state::NOMODE);

			}
		}
		//Save the level
		else if (input_->IsButtonPressed(ABFW_SONY_CTRL_START))
		{
			if (player_placed && goal_placed && wall_placed)
			{
				if (current_mode != placement_state::NOMODE)
				{
					game_objects.pop_back();
				}
			}

			SaveLevel();

			return true;

		}
		//Exit the editor
		else if (input_->IsButtonPressed(ABFW_SONY_CTRL_SELECT))
		{
			return true;
		}

		if (input_->IsAnyButtonPressed())
		{
			deltaTime = 0.0f;
		}
	}

	//Object scaling
	if (input_->IsButtonPressed(ABFW_SONY_CTRL_R2))
	{
		if (game_objects.size() > 0)
		{
			if (input_->IsButtonPressed(ABFW_SONY_CTRL_RIGHT))
			{
				game_objects.back()->set_width(game_objects.back()->width() + 2);
			}
			else if (input_->IsButtonPressed(ABFW_SONY_CTRL_LEFT))
			{
				if (game_objects.back()->width() > 2)
				{
					game_objects.back()->set_width(game_objects.back()->width() - 2);
				}
			}

			if (input_->IsButtonPressed(ABFW_SONY_CTRL_UP))
			{
				game_objects.back()->set_height(game_objects.back()->height() + 2);
			}
			else if (input_->IsButtonPressed(ABFW_SONY_CTRL_DOWN))
			{
				if (game_objects.back()->height() > 2)
				{
					game_objects.back()->set_height(game_objects.back()->height() - 2);
				}
			}
		}
	}

	//Object moving
	float x, y;
	input_->LeftStickMagnitude(x, y);

	if (x > 0.0f)
	{
		current_position.x += 2;
		game_objects.back()->SetPosition(current_position.x, current_position.y);
	}
	else if (x <0.0f)
	{
		current_position.x -= 2;
		game_objects.back()->SetPosition(current_position.x, current_position.y);
	}

	if (y < 0.0f)
	{
		current_position.y += 2;
		game_objects.back()->SetPosition(current_position.x, current_position.y);
	}
	else if (y > 0.0f)
	{
		current_position.y -= 2;
		game_objects.back()->SetPosition(current_position.x, current_position.y);
	}

	if (game_objects.size() > 0)
	{
		for (auto o : game_objects)
		{
			o->CreationFrame();
		}
	}

	return false;
}

void SceneCreatorClass::ClearScene()
{
	while (game_objects.size() > 0)
	{
		game_objects.pop_back();
	}
	player_placed = false;
	goal_placed = false;
	wall_placed = false;

	current_position = abfw::Vector2(platform_->width() / 2, platform_->height() / 2);
	ChangeModeNoDelete(placement_state::WALL);
}

void SceneCreatorClass::ChangeMode(placement_state mode)
{
	GameObject* objectPtr = NULL;

	game_objects.pop_back();

	current_mode = mode;

	switch (mode)
	{
	case(placement_state::WALL) :
		
		objectPtr = NULL;

		objectPtr = new GameObject;

		objectPtr->Initialize(current_position.x, current_position.y, 32, 32, "wall", 0xffffffff, platform_height_);

		game_objects.push_back(objectPtr);
		
		break;

	case(placement_state::ZONE):
		objectPtr = NULL;

		objectPtr = new GameObject;

		objectPtr->Initialize(current_position.x, current_position.y, 32, 32, "zone", 0xff0000ff, platform_height_);

		game_objects.push_back(objectPtr);

		break;

	case(placement_state::PLAYER):
		objectPtr = NULL;

		objectPtr = new GameObject;

		objectPtr->Initialize(current_position.x, current_position.y, 32, 32, "player", 0xffff0000, platform_height_);

		game_objects.push_back(objectPtr);

		break;

	case(placement_state::GOAL):
		objectPtr = NULL;

		objectPtr = new GameObject;

		objectPtr->Initialize(current_position.x, current_position.y, 32, 32, "target", 0xff00ff00, platform_height_);

		game_objects.push_back(objectPtr);

		break;

	case(placement_state::NOMODE) :
		break;
	}
}

void SceneCreatorClass::ChangeModeNoDelete(placement_state mode)
{
	GameObject* objectPtr = NULL;
	current_mode = mode;

	switch (mode)
	{
	case(placement_state::WALL) :

		objectPtr = NULL;

		objectPtr = new GameObject;

		objectPtr->Initialize(current_position.x, current_position.y, 32, 32, "wall", 0xffffffff, platform_height_);

		game_objects.push_back(objectPtr);

		break;

	case(placement_state::ZONE) :
		objectPtr = NULL;

		objectPtr = new GameObject;

		objectPtr->Initialize(current_position.x, current_position.y, 32, 32, "zone", 0xff0000ff, platform_height_);

		game_objects.push_back(objectPtr);

		break;

	case(placement_state::PLAYER) :
		objectPtr = NULL;

		objectPtr = new GameObject;

		objectPtr->Initialize(current_position.x, current_position.y, 32, 32, "player", 0xff0000ff, platform_height_);

		game_objects.push_back(objectPtr);

		break;

	case(placement_state::GOAL) :
		objectPtr = NULL;

		objectPtr = new GameObject;

		objectPtr->Initialize(current_position.x, current_position.y, 32, 32, "target", 0xff00ff00, platform_height_);

		game_objects.push_back(objectPtr);

		break;

	case(placement_state::NOMODE) :
		break;
	}
}

bool SceneCreatorClass::SaveLevel()
{
	File* level_file = platform_->CreateFile();

	char filename[7];

	level_count_++;

	//Calculate the filename of the new file from the current level count
	int hundred, ten, one;

	hundred = (level_count_ / 100);
	ten = ((level_count_ / 100) % 10);
	one = level_count_ % 10;

	std::snprintf(filename, 200, "%d%d%d.txt", hundred, ten, one);

	filenames_.push_back(filename);

	if (!level_file->WriteOpen(filename))
	{
		return false;
	}

	//Create the buffer that will store the data to be written to the file
	int buffer_size = (6 + (23 * game_objects.size()));
	char* level_buffer = new char[buffer_size];

	level_buffer[0] = (int)(game_objects.size() / 100) + 48;
	level_buffer[1] = (int)((game_objects.size() % 100) / 10) + 48;
	level_buffer[2] = (int)(game_objects.size() % 10) + 48;
	level_buffer[3] = 13;
	level_buffer[4] = 10;
	level_buffer[5] = 13;

	int x = 0;
	for (list<GameObject*>::iterator object = game_objects.begin(); object != game_objects.end(); object++)
	{
		int buffer_position = 23 * x;
		level_buffer[6 + buffer_position] = 10;
		//Add the object position to the buffer
		level_buffer[7 + buffer_position] = (int)((*object)->GetPosition().x / 100) + 48;
		level_buffer[8 + buffer_position] = (int)(((int)(*object)->GetPosition().x % 100) / 10) + 48;
		level_buffer[9 + buffer_position] = (int)((int)(*object)->GetPosition().x % 10) + 48;
		level_buffer[10 + buffer_position] = ',';
		level_buffer[11 + buffer_position] = (int)((*object)->GetPosition().y / 100) + 48;
		level_buffer[12 + buffer_position] = (int)(((int)(*object)->GetPosition().y % 100) / 10) + 48;
		level_buffer[13 + buffer_position] = (int)((int)(*object)->GetPosition().y % 10) + 48;
		level_buffer[14 + buffer_position] = ',';
		level_buffer[15 + buffer_position] = (int)((*object)->GetPosition().z / 100) + 48;
		level_buffer[16 + buffer_position] = (int)(((int)(*object)->GetPosition().z % 100) / 10) + 48;
		level_buffer[17 + buffer_position] = (int)((int)(*object)->GetPosition().z % 10) + 48;
		level_buffer[18 + buffer_position] = ',';
		//Add the object scale to the buffer
		level_buffer[19 + buffer_position] = (int)((*object)->width() / 100) + 48;
		level_buffer[20 + buffer_position] = (int)(((int)(*object)->width() % 100) / 10) + 48;
		level_buffer[21 + buffer_position] = (int)((int)(*object)->width() % 10) + 48;
		level_buffer[22 + buffer_position] = ',';
		level_buffer[23 + buffer_position] = (int)((*object)->height() / 100) + 48;
		level_buffer[24 + buffer_position] = (int)(((int)(*object)->height() % 100) / 10) + 48;
		level_buffer[25 + buffer_position] = (int)((int)(*object)->height() % 10) + 48;
		level_buffer[26 + buffer_position] = ',';
		//Add an index to represent which type of object this object is
		if ((*object)->GetPhysObjPtr()->GetObjectTag() == "wall")
		{
			level_buffer[27 + buffer_position] = 48;
		}
		else if ((*object)->GetPhysObjPtr()->GetObjectTag() == "zone")
		{
			level_buffer[27 + buffer_position] = 49;
		}
		else if ((*object)->GetPhysObjPtr()->GetObjectTag() == "player")
		{
			level_buffer[27 + buffer_position] = 50;
		}
		else if ((*object)->GetPhysObjPtr()->GetObjectTag() == "target")
		{
			level_buffer[27 + buffer_position] = 51;
		}
		level_buffer[28 + buffer_position] = 13;

		x++;
	}

	level_file->Write(level_buffer, buffer_size);

	level_file->Close();

	File* level_list_file = platform_->CreateFile();

	if (!level_list_file->WriteOpen("level_list.txt"))
	{
		return false;
	}

	int buffer_position = 0;

	//Create the buffer that will store the data to be written to the file
	buffer_size = (5 + (9 *(level_count_)));
	char* list_buffer = new char[buffer_size];

	//Add in the level count
	list_buffer[buffer_position] = hundred + '0';
	buffer_position++;
	list_buffer[buffer_position] = ten + '0';
	buffer_position++;
	list_buffer[buffer_position] = one + '0';
	buffer_position++;

	list_buffer[buffer_position] = 13;
	buffer_position++;
	list_buffer[buffer_position] = 10;
	buffer_position++;
	list_buffer[buffer_position] = 13;
	buffer_position++;

	//Write the filenames to the buffer
	for (list<string>::iterator names = filenames_.begin(); names != filenames_.end(); names++)
	{
		list_buffer[buffer_position] = 10;
		buffer_position++;
		for (int i = 0; i < 7; i++)
		{
			list_buffer[buffer_position] = (*names)[i];
			buffer_position++;
		}
		list_buffer[buffer_position] = 13;
		buffer_position++;
	}

	level_list_file->Write(list_buffer, buffer_size);

	level_list_file->Close();


	return true;
}