#include "scene_select_class.h"

SceneSelectClass::SceneSelectClass()
{
	input_ = NULL;
	platform_ = NULL;
	level_count_ = 0;
	current_selection_ = 1;
	deltaTime_ = 0.0f;
}

SceneSelectClass::SceneSelectClass(const SceneSelectClass& other)
{
}

SceneSelectClass::~SceneSelectClass()
{
}

bool SceneSelectClass::Initialize(Platform& platform, InputClass* input, const char* filename)
{
	bool result; 

	platform_ = &platform;
		
	input_ = input;

	result = LoadLevelCount();

	if (!result)
	{
		return false;
	}

	return true;
}

void SceneSelectClass::Shutdown()
{
	while (game_objects.size() > 0)
	{
		game_objects.back()->Shutdown();
		delete game_objects.back();
		game_objects.pop_back();
	}
}

int SceneSelectClass::Frame(float dt)
{
	float posX, posY;

	deltaTime_ += dt;

	if (deltaTime_ > 0.4f)
	{
		if (input_->IsButtonPressed(ABFW_SONY_CTRL_RIGHT))
		{
			if (current_selection_ < level_count_)
			{
				current_selection_++;
			}
		}
		else if (input_->IsButtonPressed(ABFW_SONY_CTRL_LEFT))
		{
			if (current_selection_ > 1)
			{
				current_selection_--;
			}
		}

		if (input_->IsButtonPressed(ABFW_SONY_CTRL_CROSS))
		{
			return current_selection_;
		}

		if (input_->IsAnyButtonPressed())
		{
			deltaTime_ = 0;
		}
	}
	
	//Touch screen check for level editor
	if (input_->IsTouchDown(0, 0))
	{
		input_->GetTouchPosition(0, 0, posX, posY);

		if (posX > ((platform_->width() / 2) - (platform_->width() / 4)) && posX < ((platform_->width() / 2) + (platform_->width() / 4)))
		{
			if (posY >(platform_->height() / 2) && posY < (platform_->height() / 2) + 200)
			{
				return -1;
			}
		}
	}
	

	return 0;
}

void SceneSelectClass::Render(abfw::SpriteRendererVita* renderer_, abfw::Font& font)
{
	for (list<GameObject*>::iterator object = game_objects.begin(); object != game_objects.end(); object++)
	{
		renderer_->DrawSprite(*(*object));
	}

	char level[20];
	std::snprintf(level, 200, "Level %d", current_selection_);
	
	font.RenderText(renderer_, abfw::Vector3((platform_->width() / 2), (platform_->height() / 8), -0.9f), 10.0f, 0xff0000ff, abfw::TJ_CENTRE, level);
	font.RenderText(renderer_, abfw::Vector3((platform_->width() / 2), ((platform_->height() / 2) + 100), -0.9f), 5.0f, 0xff0000ff, abfw::TJ_CENTRE, "Level Editor");
}


bool SceneSelectClass::LoadLevelCount()
{
	int bufferPosition;
	int hundreds, tens, ones;
	File* level_file = platform_->CreateFile();

	//Open the level list file
	bool success = level_file->Open("level_list.txt");
	if (success)
	{
		char* buffer = NULL;
		Int32 file_size;
		success = level_file->GetSize(file_size);
		if (success)
		{
			//Read the file into a buffer
			buffer = new char[file_size];
			Int32 bytes_read;
			success = level_file->Read(buffer, file_size, bytes_read);

			if (success)
				level_file->Close();

			if (success)
			{
				//Read the level count
				bufferPosition = 0;
				hundreds = buffer[bufferPosition] - '0';
				bufferPosition++;
				tens = buffer[bufferPosition] - '0';
				bufferPosition++;
				ones = buffer[bufferPosition] - '0';

				level_count_ = (hundreds * 100) + (tens * 10) + ones;

				return true;
			}
		}
	}

	return false;
}