#ifndef _SPRITE_APPLICATION_H
#define _SPRITE_APPLICATION_H

#include <system/application.h>
#include <graphics/sprite.h>
#include <graphics/font.h>
#include <input/sony_controller_input_manager.h>
#include <audio/vita/audio_manager_vita.h>
#include <Box2D/Box2D.h>

#include "inputclass.h"
#include "sceneclass.h"
#include "title_scene_class.h"
#include "pause_menu_scene.h"
#include "options_storage.h"
#include "options_scene.h"
#include "static_scene.h"

namespace abfw
{
	class Platform;
	class Texture;
}

enum GameState
{
	TITLE_SCREEN, IN_GAME, PAUSE_MENU, OPTIONS_MENU, GAME_WIN, GAME_LOSE
};

class SpriteApplication : public abfw::Application
{
public:
	SpriteApplication(abfw::Platform& platform);
	~SpriteApplication();
	void Init();
	void CleanUp();
	bool Update(float ticks);
	void Render();

private:
	abfw::Font font_;
	abfw::SpriteRenderer* sprite_renderer_;
	abfw::AudioManagerVita* audio_manager_;
	float frame_rate_;
	float frame_time_;
	InputClass* input_;
	SceneClass* scene_;
	TitleSceneClass title_scene_;
	PauseMenuSceneClass pause_menu_;
	GameState game_state_ = TITLE_SCREEN;
	OptionsStorage options_storage_;
	OptionsScene options_menu_;
	StaticScene win_scene_;
	StaticScene lose_scene_;


};

#endif // _SPRITE_APPLICATION_H