#include "title_scene_class.h"

TitleSceneClass::TitleSceneClass()
{
	input_ = NULL;
	platform_ = NULL;
	background_image_ = NULL;
}

TitleSceneClass::TitleSceneClass(const TitleSceneClass& other)
{
}

TitleSceneClass::~TitleSceneClass()
{
}

bool TitleSceneClass::Initialize(Platform* platform, abfw::AudioManagerVita* audio_manager, InputClass* input, abfw::SpriteRenderer* renderer, abfw::Font& font, OptionsStorage* options_storage, const char* filename)
{
	platform_ = platform;

	input_ = input;

	audio_manager_ = audio_manager;
	
	options_storage_ = options_storage;

	//load the music
	ambience_music_.Initialize(audio_manager_, platform_, NULL);

	ambience_music_.LoadSoundData("Blind_Shift.wav");

	ambience_music_.PlaySound();

	//load ui objects
	play_button_ = new UIButton;

	play_button_->Initialize(*platform, "play_button_anim.png", input_, 480, 275, 300, 150, true);

	ui_objects_.push_back(play_button_);
	
	options_button_ = new UIButton;

	options_button_->Initialize(*platform, "options_button_anim.png", input_, 480, 435, 300, 150, true);

	ui_objects_.push_back(options_button_);

	UIText* text = new UIText;

	text->Initialize("Space Jam!", 0xFF990099, 480, 10, 6);

	ui_objects_.push_back(text);

	background_image_ = new AnimatedSprite;

	background_image_->LoadTexture("background.png", *platform);
	background_image_->set_width(1024);
	background_image_->set_height(1024);
	background_image_->set_position(abfw::Vector3(platform->width() / 2, platform->height() / 2, 0.9f));

	return true;
}

void TitleSceneClass::RestartMusic()
{
	ambience_music_.PlaySound();
}

void TitleSceneClass::Shutdown()
{
	platform_ = NULL;
	input_ = NULL;
}

int TitleSceneClass::Frame(float dt)
{
	float new_uv_x = background_image_->uv_position().x + (dt/2.0f);

	if (new_uv_x >= 1.0f)
		new_uv_x -= 1.0f;

	background_image_->set_uv_position(abfw::Vector2(new_uv_x, 0.0f));

	if (play_button_->Frame(dt))
		return 1;

	if (options_button_->Frame(dt))
		return 3;

	return 0;
}
