#include "ui_value_bar.h"

UIValueBar::~UIValueBar()
{
}

void UIValueBar::Initialize(abfw::Platform& platform, char* textureFilename, Justification justification, int max_size, int constant_position, int altPos, int* targetValue, int* maxValue)
{
	if (textureFilename)
	{
		LoadTexture(textureFilename, platform);
	}

	justification_ = justification;

	max_size_ = max_size;
	constant_position_ = constant_position;
	targetValue_ = targetValue;
	maxValue_ = maxValue;
	
	switch (justification_)
	{
	case LEFT:
		middle_position_ = constant_position_ + (max_size_ / 2);
		set_position(middle_position_, altPos, -0.9f);
		set_height(16);
		break;
	case RIGHT:
		middle_position_ = constant_position_ - (max_size_ / 2);
		set_position(middle_position_, altPos, -0.9f);
		set_height(16);
		break;
	case TOP:
		middle_position_ = constant_position_ - (max_size_ / 2);
		set_position(altPos, middle_position_, -0.9f);
		set_width(16);
		break;
	case BOTTOM:
		middle_position_ = constant_position_ + (max_size_ / 2);
		set_position(altPos, middle_position_, -0.9f);
		set_width(16);
		break;
	}
	
	
}

bool UIValueBar::Frame(float dt)
{
	float size;

	size = 1.0f;

	if (*targetValue_ < *maxValue_)
		size = (float)*targetValue_ / (float)*maxValue_;

	switch (justification_)
	{
	case LEFT:
		set_width(max_size_ * size);
		middle_position_ =(int)(constant_position_ + (width_ / 2)); 
		set_position(middle_position_, position_.y, -0.9f);
		set_uv_width(size);
		break;
	case RIGHT:
		set_width(max_size_ * size);
		middle_position_ = (int)(constant_position_ - (width_ / 2));
		set_position(middle_position_, position_.y, -0.9f);
		set_uv_width(size);
		set_uv_position(abfw::Vector2(1.0f - size, 0.0f));
		break;
	case TOP:
		set_height(max_size_ * size);
		middle_position_ = (int)(constant_position_ + (height_ / 2));
		set_position(position_.x, middle_position_, -0.9f);
		set_uv_height(size);
		break;
	case BOTTOM:
		set_height(max_size_ * size);
		middle_position_ = (int)(constant_position_ - (height_ / 2));
		set_position(position_.x, middle_position_, -0.9f);
		set_uv_height(size);
		set_uv_position(abfw::Vector2(0.0f, 1.0f - size));
		break;
	}	

	return false;
}

void UIValueBar::Render(abfw::SpriteRenderer* renderer_, abfw::Font& font)
{
	renderer_->DrawSprite(*this);

	switch (justification_)
	{
	case LEFT:
		font.RenderText(renderer_, abfw::Vector3(constant_position_ + max_size_ + 4, position_.y - 16, -0.9f), 1.0f, 0xff00ffff, abfw::TJ_LEFT, "%d / %d", *targetValue_, *maxValue_);
		break;
	case RIGHT:
		font.RenderText(renderer_, abfw::Vector3(constant_position_ - max_size_ - 4, position_.y - 16, -0.9f), 1.0f, 0xff00ffff, abfw::TJ_LEFT, "%d / %d", *targetValue_, *maxValue_);
		break;
	case TOP:
		font.RenderText(renderer_, abfw::Vector3(position_.x, constant_position_  - 16, -0.9f), 1.0f, 0xff00ffff, abfw::TJ_CENTRE, "%d / %d", *targetValue_, *maxValue_);
		break;
	case BOTTOM:
		font.RenderText(renderer_, abfw::Vector3(position_.x, constant_position_, -0.9f), 1.0f, 0xff00ffff, abfw::TJ_CENTRE, "%d / %d", *targetValue_, *maxValue_);
		break;

	}
}