#include "weapon_object.h"

#include "sceneclass.h"

void WeaponObject::Frame(b2Vec2 position, b2Vec2 camera_position, float rotation, float dt)
{
	world_position_ = b2Vec2(0.0f, 0.0f);
	world_position_ = position;
	weapon_rotation_ = rotation;

	b2Vec2 screen_position = world_position_ - camera_position;

	set_rotation(-(object_body_->GetAngle()));

	set_position(abfw::Vector3(BOX2D_ABFW_POS_X(screen_position.x), (BOX2D_ABFW_POS_Y(screen_position.y) - (plat_height / 2)), 0.f));

	UpdateAnimation(dt);
	cooldown_time_ += dt;
	barrage_time_ += dt;

	if (firing_)
	{
		if (barrage_time_ >= barrage_rate_)
		{
			barrages_fired_++;
			if (barrages_fired_ == num_barrages_)
			{
				firing_ = false;
			}

			barrage_time_ -= barrage_rate_;
			shot_sound_effect_.PlaySound();

			SceneClass* scene = static_cast<SceneClass*>(owner_scene_);
			float rotate = object_body_->GetAngle();
			float angle_between;
			int projectile_count = num_projectiles_;

			if (projectile_count % 2 != 0)
			{
				ProjectileObject* projectile = (ProjectileObject*)(scene->InstantiateObject(projectile_, world_position_.x, world_position_.y, weapon_rotation_));
				projectile->owner_scene_ = owner_scene_;
				b2Vec2 directionalVector = b2Vec2(-sin(weapon_rotation_), cos(weapon_rotation_));
				projectile->GetObjectBody()->ApplyLinearImpulse(b2Vec2(directionalVector.x * weapon_force_, directionalVector.y * weapon_force_), b2Vec2(0, 0));
				projectile_count--;
			}

			angle_between = (projectile_angle_ * (b2_pi/180.0f)) / (float)projectile_count;

			int angle_count;
			int flipper = -1;
			float firing_angle;

			//Create a copy of the projectile(s) and apply forces to them
			for (int i = 0; i < projectile_count; i++)
			{
				angle_count = ((i / 2) + 1) * flipper;
				firing_angle = weapon_rotation_ + (angle_between * angle_count);
				ProjectileObject* projectile = (ProjectileObject*)(scene->InstantiateObject(projectile_, world_position_.x, world_position_.y, firing_angle));
				projectile->owner_scene_ = owner_scene_;
				b2Vec2 directionalVector = b2Vec2(-sin(firing_angle), cos(firing_angle));
				projectile->GetObjectBody()->ApplyLinearImpulse(b2Vec2(directionalVector.x * weapon_force_, directionalVector.y * weapon_force_), b2Vec2(0, 0));
				flipper = -flipper;
			}
		}
	}

}

void WeaponObject::Fire()
{
	if (cooldown_time_ >= (1.0f/fire_rate_))
	{
		firing_ = true;
		cooldown_time_ = 0.0f;
		barrage_time_ = barrage_rate_;
		barrages_fired_ = 0;
	}
}

void WeaponObject::LoadUniqueData(char* buffer, int& bufferPosition)
{
	char* projectileFilename;
	char* sfxFilename;
	int target_filter;

	ParseCharArray(buffer, bufferPosition, projectileFilename);
	bufferPosition += 2;

	ParseCharArray(buffer, bufferPosition, sfxFilename);
	bufferPosition += 2;

	ParseInt(buffer, bufferPosition, num_barrages_);
	bufferPosition += 2;

	ParseInt(buffer, bufferPosition, num_projectiles_);
	bufferPosition += 2;

	ParseFloat(buffer, bufferPosition, projectile_angle_);
	bufferPosition += 2;

	ParseFloat(buffer, bufferPosition, barrage_rate_);
	bufferPosition += 2;

	ParseFloat(buffer, bufferPosition, fire_rate_);
	bufferPosition += 2;

	ParseFloat(buffer, bufferPosition, weapon_force_);
	bufferPosition += 2;

	ParseInt(buffer, bufferPosition, target_filter);
	bufferPosition += 2;

	target_filter_ = target_filter;

	shot_sound_effect_.Initialize(audio_manager_, platform_, NULL);

	shot_sound_effect_.LoadSoundData(sfxFilename);

	shot_sound_effect_.SetSampleVolume(0.1f);

	projectile_ = new ProjectileObject;

	projectile_->LoadDataFile(projectileFilename, platform_, object_body_->GetWorld(), audio_manager_, input_, options_storage_, 0.0f, 0.0f, 0.0f);

	projectile_->SetTargetFilter(target_filter_);

	projectile_->visible_ = false;

	barrages_fired_ = 0;
	firing_ = false;


}

void WeaponObject::Shutdown()
{
	audio_manager_->UnloadSample(shot_sound_effect_.GetId());
	delete projectile_;
	projectile_ = NULL;
}